@@ -11,9 +11,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import logging
-
 from django.conf import settings
+from keystoneauth1 import identity
+from keystoneauth1 import session
 from saharaclient.api.base import APIException
 from saharaclient.api.base import Page
 from saharaclient import client as api_client
@@ -26,17 +26,24 @@ from openstack_dashboard.api import base
 from sahara_dashboard import utils as u
 
 
-LOG = logging.getLogger(__name__)
-
 # "type" of Sahara service registered in keystone
 SAHARA_SERVICE = 'data-processing'
-# Sahara service_type registered in Juno
-SAHARA_SERVICE_FALLBACK = 'data_processing'
 
-SAHARA_AUTO_IP_ALLOCATION_ENABLED = getattr(
+try:
+    SAHARA_FLOATING_IP_DISABLED = getattr(
+        settings,
+        'SAHARA_FLOATING_IP_DISABLED')
+except AttributeError:
+    SAHARA_FLOATING_IP_DISABLED = getattr(
+        settings,
+        'SAHARA_AUTO_IP_ALLOCATION_ENABLED',
+        False)
+
+SAHARA_VERIFICATION_DISABLED = getattr(
     settings,
-    'SAHARA_AUTO_IP_ALLOCATION_ENABLED',
+    'SAHARA_VERIFICATION_DISABLED',
     False)
+
 VERSIONS = base.APIVersionManager(
     SAHARA_SERVICE,
     preferred_version=getattr(settings,
@@ -44,6 +51,8 @@ VERSIONS = base.APIVersionManager(
                               {}).get(SAHARA_SERVICE, 1.1))
 VERSIONS.load_supported_version(1.1, {"client": api_client,
                                       "version": 1.1})
+VERSIONS.load_supported_version(2, {"client": api_client,
+                                    "version": 2})
 
 SAHARA_PAGE_SIZE = 15
 
@@ -69,7 +78,7 @@ def safe_call(func, *args, **kwargs):
     """Call a function ignoring Not Found error
 
     This method is supposed to be used only for safe retrieving Sahara
-    objects. If the object is no longer available the None should be
+    objects. If the object is no longer available, then None should be
     returned.
 
     """
@@ -84,23 +93,20 @@ def safe_call(func, *args, **kwargs):
 
 @memoized
 def client(request):
-    try:
-        service_type = SAHARA_SERVICE
-        sahara_url = base.url_for(request, service_type)
-    except exceptions.ServiceCatalogException:
-        # if no endpoint found, fallback to the old service_type
-        service_type = SAHARA_SERVICE_FALLBACK
-        sahara_url = base.url_for(request, service_type)
-
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
     cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
+    auth = identity.Token(auth_url=request.user.endpoint,
+                          token=request.user.token.id,
+                          project_id=request.user.project_id)
+    verify = False
+    if cacert:
+        verify = cacert
+    elif not insecure:
+        verify = True
+    sess = session.Session(auth=auth, verify=verify)
     return api_client.Client(VERSIONS.get_active_version()["version"],
-                             sahara_url=sahara_url,
-                             service_type=service_type,
-                             project_id=request.user.project_id,
-                             input_auth_token=request.user.token.id,
-                             insecure=insecure,
-                             cacert=cacert)
+                             service_type=SAHARA_SERVICE,
+                             session=sess)
 
 
 def prepare_acl_update_dict(is_public=None, is_protected=None):
@@ -145,8 +151,7 @@ def plugin_get(request, plugin_name):
 
 def plugin_get_version_details(request, plugin_name, hadoop_version):
     return client(request).plugins.get_version_details(
-        plugin_name=plugin_name,
-        hadoop_version=hadoop_version)
+        plugin_name, hadoop_version)
 
 
 def nodegroup_template_create(request, name, plugin_name, hadoop_version,
@@ -164,11 +169,16 @@ def nodegroup_template_create(request, name, plugin_name, hadoop_version,
                               use_autoconfig=None,
                               shares=None,
                               is_public=None,
-                              is_protected=None):
-    return client(request).node_group_templates.create(
+                              is_protected=None,
+                              volume_mount_prefix=None,
+                              boot_from_volume=None,
+                              boot_volume_type=None,
+                              boot_volume_availability_zone=None,
+                              boot_volume_local_to_instance=None):
+
+    payload = dict(
         name=name,
         plugin_name=plugin_name,
-        hadoop_version=hadoop_version,
         flavor_id=flavor_id,
         description=description,
         volumes_per_node=volumes_per_node,
@@ -187,7 +197,23 @@ def nodegroup_template_create(request, name, plugin_name, hadoop_version,
         use_autoconfig=use_autoconfig,
         shares=shares,
         is_public=is_public,
-        is_protected=is_protected)
+        is_protected=is_protected,
+        volume_mount_prefix=volume_mount_prefix)
+
+    if VERSIONS.active == '2':
+        payload['plugin_version'] = hadoop_version
+        payload['boot_from_volume'] = boot_from_volume
+        payload['boot_volume_type'] = boot_volume_type
+        payload['boot_volume_availability_zone'] = (
+            boot_volume_availability_zone
+        )
+        payload['boot_volume_local_to_instance'] = (
+            boot_volume_local_to_instance
+        )
+    else:
+        payload['hadoop_version'] = hadoop_version
+
+    return client(request).node_group_templates.create(**payload)
 
 
 def nodegroup_template_list(request, search_opts=None,
@@ -202,6 +228,8 @@ def nodegroup_template_get(request, ngt_id):
 
 
 def nodegroup_template_find(request, **kwargs):
+    if "hadoop_version" in kwargs and VERSIONS.active == '2':
+        kwargs["plugin_version"] = kwargs.pop("hadoop_version")
     return client(request).node_group_templates.find(**kwargs)
 
 
@@ -224,12 +252,16 @@ def nodegroup_template_update(request, ngt_id, name, plugin_name,
                               shares=None,
                               is_protected=None,
                               is_public=None,
-                              image_id=None):
-    return client(request).node_group_templates.update(
+                              image_id=None,
+                              boot_from_volume=None,
+                              boot_volume_type=None,
+                              boot_volume_availability_zone=None,
+                              boot_volume_local_to_instance=None):
+
+    payload = dict(
         ng_template_id=ngt_id,
         name=name,
         plugin_name=plugin_name,
-        hadoop_version=hadoop_version,
         flavor_id=flavor_id,
         description=description,
         volumes_per_node=volumes_per_node,
@@ -250,6 +282,22 @@ def nodegroup_template_update(request, ngt_id, name, plugin_name,
         is_protected=is_protected,
         image_id=image_id)
 
+    if VERSIONS.active == '2':
+        payload['plugin_version'] = hadoop_version
+        payload['boot_from_volume'] = boot_from_volume
+        payload['boot_volume_type'] = boot_volume_type
+        payload['boot_volume_availability_zone'] = (
+            boot_volume_availability_zone
+        )
+        payload['boot_volume_local_to_instance'] = (
+            boot_volume_local_to_instance
+        )
+
+    else:
+        payload['hadoop_version'] = hadoop_version
+
+    return client(request).node_group_templates.update(**payload)
+
 
 def nodegroup_update_acl_rules(request, nid,
                                is_public=None, is_protected=None):
@@ -257,16 +305,19 @@ def nodegroup_update_acl_rules(request, nid,
         nid, **prepare_acl_update_dict(is_public, is_protected))
 
 
+def nodegroup_template_export(request, object_id):
+    return client(request).node_group_templates.export(object_id)
+
+
 def cluster_template_create(request, name, plugin_name, hadoop_version,
                             description=None, cluster_configs=None,
                             node_groups=None, anti_affinity=None,
                             net_id=None, use_autoconfig=None, shares=None,
                             is_public=None, is_protected=None,
                             domain_name=None):
-    return client(request).cluster_templates.create(
+    payload = dict(
         name=name,
         plugin_name=plugin_name,
-        hadoop_version=hadoop_version,
         description=description,
         cluster_configs=cluster_configs,
         node_groups=node_groups,
@@ -278,6 +329,11 @@ def cluster_template_create(request, name, plugin_name, hadoop_version,
         is_protected=is_protected,
         domain_name=domain_name
     )
+    if VERSIONS.active == '2':
+        payload['plugin_version'] = hadoop_version
+    else:
+        payload['hadoop_version'] = hadoop_version
+    return client(request).cluster_templates.create(**payload)
 
 
 def cluster_template_list(request, search_opts=None, marker=None, limit=None):
@@ -304,11 +360,10 @@ def cluster_template_update(request, ct_id, name, plugin_name,
                             is_public=None, is_protected=None,
                             domain_name=None):
     try:
-        template = client(request).cluster_templates.update(
+        payload = dict(
             cluster_template_id=ct_id,
             name=name,
             plugin_name=plugin_name,
-            hadoop_version=hadoop_version,
             description=description,
             cluster_configs=cluster_configs,
             node_groups=node_groups,
@@ -320,6 +375,11 @@ def cluster_template_update(request, ct_id, name, plugin_name,
             is_protected=is_protected,
             domain_name=domain_name
         )
+        if VERSIONS.active == '2':
+            payload['plugin_version'] = hadoop_version
+        else:
+            payload['hadoop_version'] = hadoop_version
+        template = client(request).cluster_templates.update(**payload)
 
     except APIException as e:
         raise exceptions.Conflict(e)
@@ -332,16 +392,19 @@ def cluster_template_update_acl_rules(request, ct_id,
         ct_id, **prepare_acl_update_dict(is_public, is_protected))
 
 
+def cluster_template_export(request, object_id):
+    return client(request).cluster_templates.export(object_id)
+
+
 def cluster_create(request, name, plugin_name, hadoop_version,
                    cluster_template_id=None, default_image_id=None,
                    is_transient=None, description=None, cluster_configs=None,
                    node_groups=None, user_keypair_id=None, anti_affinity=None,
                    net_id=None, count=None, use_autoconfig=None,
                    is_public=None, is_protected=None):
-    return client(request).clusters.create(
+    payload = dict(
         name=name,
         plugin_name=plugin_name,
-        hadoop_version=hadoop_version,
         cluster_template_id=cluster_template_id,
         default_image_id=default_image_id,
         is_transient=is_transient,
@@ -355,6 +418,11 @@ def cluster_create(request, name, plugin_name, hadoop_version,
         use_autoconfig=use_autoconfig,
         is_public=is_public,
         is_protected=is_protected)
+    if VERSIONS.active == '2':
+        payload['plugin_version'] = hadoop_version
+    else:
+        payload['hadoop_version'] = hadoop_version
+    return client(request).clusters.create(**payload)
 
 
 def cluster_scale(request, cluster_id, scale_object):
@@ -383,6 +451,10 @@ def cluster_delete(request, cluster_id):
     client(request).clusters.delete(cluster_id=cluster_id)
 
 
+def cluster_force_delete(request, cluster_id):
+    client(request).clusters.force_delete(cluster_id=cluster_id)
+
+
 def cluster_update(request, cluster_id, name=None, description=None,
                    is_public=None, is_protected=None, shares=None):
     return client(request).clusters.update(cluster_id,
@@ -405,16 +477,18 @@ def cluster_update_acl_rules(request, cl_id, is_public=None,
 
 def data_source_create(request, name, description, ds_type, url,
                        credential_user=None, credential_pass=None,
-                       is_public=None, is_protected=None):
+                       is_public=None, is_protected=None,
+                       s3_credentials=None):
     return client(request).data_sources.create(
         name=name,
         description=description,
         data_source_type=ds_type,
         url=url,
-        credential_user=credential_user,
-        credential_pass=credential_pass,
+        credential_user=credential_user or None,
+        credential_pass=credential_pass or None,
         is_public=is_public,
-        is_protected=is_protected)
+        is_protected=is_protected,
+        s3_credentials=s3_credentials)
 
 
 def data_source_list(request, search_opts=None, limit=None, marker=None):
@@ -502,7 +576,12 @@ def job_binary_internal_delete(request, jbi_id):
 
 def job_create(request, name, j_type, mains, libs, description, interface,
                is_public=None, is_protected=None):
-    return client(request).jobs.create(
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'job_templates'
+    else:
+        manager = 'jobs'
+    return getattr(sahara, manager).create(
         name=name,
         type=j_type,
         mains=mains,
@@ -514,32 +593,62 @@ def job_create(request, name, j_type, mains, libs, description, interface,
 
 
 def job_update(request, job_id, is_public=None, is_protected=None):
-    return client(request).jobs.update(
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'job_templates'
+    else:
+        manager = 'jobs'
+    return getattr(sahara, manager).update(
         job_id=job_id, **prepare_acl_update_dict(is_public, is_protected))
 
 
 def job_list(request, search_opts=None, marker=None, limit=None):
     marker, limit = _update_pagination_params(marker, limit, request)
-    return client(request).jobs.list(
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'job_templates'
+    else:
+        manager = 'jobs'
+    return getattr(sahara, manager).list(
         search_opts=search_opts,
         limit=limit,
         marker=marker)
 
 
 def _job_list(request):
-    return client(request).jobs.list()
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'job_templates'
+    else:
+        manager = 'jobs'
+    return getattr(sahara, manager).list()
 
 
 def job_get(request, job_id):
-    return client(request).jobs.get(job_id=job_id)
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'job_templates'
+    else:
+        manager = 'jobs'
+    return getattr(sahara, manager).get(job_id=job_id)
 
 
 def job_delete(request, job_id):
-    client(request).jobs.delete(job_id=job_id)
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'job_templates'
+    else:
+        manager = 'jobs'
+    getattr(sahara, manager).delete(job_id=job_id)
 
 
 def job_get_configs(request, job_type):
-    return client(request).jobs.get_configs(job_type=job_type)
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'job_templates'
+    else:
+        manager = 'jobs'
+    return getattr(sahara, manager).get_configs(job_type=job_type)
 
 
 def job_execution_create(request, job_id, cluster_id,
@@ -549,8 +658,13 @@ def job_execution_create(request, job_id, cluster_id,
         input_id = None
     if output_id in [None, "", "None"]:
         output_id = None
-    return client(request).job_executions.create(
-        job_id=job_id,
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'jobs'
+    else:
+        manager = 'job_executions'
+    return getattr(sahara, manager).create(
+        job_id,
         cluster_id=cluster_id,
         input_id=input_id,
         output_id=output_id,
@@ -562,9 +676,14 @@ def job_execution_create(request, job_id, cluster_id,
 
 
 def job_execution_update(request, jbe_id, is_public=None, is_protected=None):
-    return client(request).job_executions.update(job_execution_id=jbe_id,
-                                                 **prepare_acl_update_dict(
-                                                     is_public, is_protected))
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'jobs'
+    else:
+        manager = 'job_executions'
+    return getattr(sahara, manager).update(jbe_id,
+                                           **prepare_acl_update_dict(
+                                               is_public, is_protected))
 
 
 def _resolve_job_execution_names(job_execution, cluster=None,
@@ -583,21 +702,32 @@ def _resolve_job_execution_names(job_execution, cluster=None,
 
 def job_execution_list(request, search_opts=None, marker=None, limit=None):
     marker, limit = _update_pagination_params(marker, limit, request)
-    job_execution_list = client(request).job_executions.list(
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'jobs'
+    else:
+        manager = 'job_executions'
+    job_execution_list = getattr(sahara, manager).list(
         search_opts=search_opts, limit=limit,
         marker=marker)
 
     new_request = u.delete_pagination_params_from_request(
         request, save_limit=False)
 
-    job_dict = dict((j.id, j) for j in _job_list(new_request))
-    cluster_dict = dict((c.id, c) for c in _cluster_list(new_request))
+    job_dict = {j.id: j for j in _job_list(new_request)}
+    cluster_dict = {c.id: c for c in _cluster_list(new_request)}
+
+    def _find_jt_id(jex_obj):
+        try:
+            return jex_obj.job_template_id  # typical APIv2
+        except AttributeError:
+            return jex_obj.job_id  # APIv1.1, older APIv2
 
     resolved_job_execution_list = [
         _resolve_job_execution_names(
             job_execution,
             cluster_dict.get(job_execution.cluster_id),
-            job_dict.get(job_execution.job_id))
+            job_dict.get(_find_jt_id(job_execution)))
         for job_execution in job_execution_list
     ]
 
@@ -606,15 +736,32 @@ def job_execution_list(request, search_opts=None, marker=None, limit=None):
 
 
 def job_execution_get(request, jex_id):
-    jex = client(request).job_executions.get(obj_id=jex_id)
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        je_manager = 'jobs'
+        jt_manager = 'job_templates'
+    else:
+        je_manager = 'job_executions'
+        jt_manager = 'jobs'
+
+    jex = getattr(sahara, je_manager).get(obj_id=jex_id)
     cluster = safe_call(client(request).clusters.get, jex.cluster_id)
-    job = safe_call(client(request).jobs.get, jex.job_id)
+    try:
+        jt_id = jex.job_template_id  # typical APIv2
+    except AttributeError:
+        jt_id = jex.job_id  # APIv1.1, older APIv2
+    job = safe_call(getattr(sahara, jt_manager).get, jt_id)
 
     return _resolve_job_execution_names(jex, cluster, job)
 
 
 def job_execution_delete(request, jex_id):
-    client(request).job_executions.delete(obj_id=jex_id)
+    sahara = client(request)
+    if VERSIONS.active == '2':
+        manager = 'jobs'
+    else:
+        manager = 'job_executions'
+    getattr(sahara, manager).delete(obj_id=jex_id)
 
 
 def job_types_list(request):
