@@ -45,9 +45,24 @@ urlpatterns = [url(r'^$', views.IndexView.as_view(), name='index'),
                url(r'^configure-cluster-template$',
                    ct_views.ConfigureClusterTemplateView.as_view(),
                    name='configure-cluster-template'),
+               url(r'^import-cluster-template-file$',
+                   ct_views.ImportClusterTemplateFileView.as_view(),
+                   name='import-cluster-template-file'),
+               url(r'^import-cluster-template-name$',
+                   ct_views.ImportClusterTemplateNameView.as_view(),
+                   name='import-cluster-template-name'),
+               url(r'^import-cluster-template-nodegroups$',
+                   ct_views.ImportClusterTemplateNodegroupsView.as_view(),
+                   name='import-cluster-template-nodegroups'),
                url(r'^configure-nodegroup-template$',
                    ngt_views.ConfigureNodegroupTemplateView.as_view(),
                    name='configure-nodegroup-template'),
+               url(r'^import-nodegroup-template-file$',
+                   ngt_views.ImportNodegroupTemplateFileView.as_view(),
+                   name='import-nodegroup-template-file'),
+               url(r'^import-nodegroup-template-details$',
+                   ngt_views.ImportNodegroupTemplateDetailsView.as_view(),
+                   name='import-nodegroup-template-details'),
                url(r'^cluster-template/(?P<template_id>[^/]+)$',
                    ct_views.ClusterTemplateDetailsView.as_view(),
                    name='ct-details'),
@@ -91,6 +106,9 @@ urlpatterns = [url(r'^$', views.IndexView.as_view(), name='index'),
                url(r'^cluster_guide/(?P<reset_cluster_guide>[^/]+)/$',
                    cluster_guide_views.ResetClusterGuideView.as_view(),
                    name='reset_cluster_guide'),
+               url(r'^image_register/$',
+                   cluster_guide_views.ImageRegisterView.as_view(),
+                   name='image_register'),
                url(r'^plugin_select$',
                    cluster_guide_views.PluginSelectView.as_view(),
                    name='plugin_select'),
