@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 
 from sahara_dashboard.test import helpers as test
 
